@@ -50,9 +50,6 @@ class GLDependencySystem(SystemDependency):
             # FIXME: Detect version using self.clib_compiler
             return
 
-    def log_tried(self) -> str:
-        return 'system'
-
 class GnuStepDependency(ConfigToolDependency):
 
     tools = ['gnustep-config']
@@ -237,9 +234,6 @@ class VulkanDependencySystem(SystemDependency):
                     self.link_args.append(lib)
                 return
 
-    def log_tried(self) -> str:
-        return 'system'
-
 gl_factory = DependencyFactory(
     'gl',
     [DependencyMethods.PKGCONFIG, DependencyMethods.SYSTEM],
