@@ -20,12 +20,13 @@ from collections import OrderedDict
 from mesonbuild import mesonlib
 from mesonbuild import mlog, build
 from mesonbuild.coredata import MesonException
-from . import ModuleReturnValue
+from . import ModuleReturnValue, ModuleInfo
 from . import ExtensionModule
-from . import get_include_args
-from ..dependencies import Dependency, InternalDependency, ExternalProgram
-from ..interpreterbase import FeatureNew, InvalidArguments, noPosargs, noKwargs
+from ..dependencies import Dependency, InternalDependency
+from ..interpreterbase import InvalidArguments, noPosargs, noKwargs, typed_kwargs, ContainerTypeInfo, KwargInfo, typed_pos_args
 from ..interpreter import CustomTargetHolder
+from ..interpreter.type_checking import NoneType
+from ..programs import ExternalProgram
 
 
 def ensure_list(value):
@@ -36,8 +37,11 @@ def ensure_list(value):
 
 MIN_HOTDOC_VERSION = '0.8.100'
 
+file_types = (str, mesonlib.File, build.CustomTarget, build.CustomTargetIndex)
+
 
 class HotdocTargetBuilder:
+
     def __init__(self, name, state, hotdoc, interpreter, kwargs):
         self.hotdoc = hotdoc
         self.build_by_default = kwargs.pop('build_by_default', False)
@@ -60,14 +64,13 @@ class HotdocTargetBuilder:
         self._dependencies = []
         self._subprojects = []
 
-    def process_known_arg(self, option, types, argname=None,
-                          value_processor=None, mandatory=False,
-                          force_list=False):
+    def process_known_arg(self, option, argname=None, value_processor=None):
         if not argname:
             argname = option.strip("-").replace("-", "_")
 
-        value, _ = self.get_value(
-            types, argname, None, value_processor, mandatory, force_list)
+        value = self.kwargs.pop(argname)
+        if value is not None and value_processor:
+            value = value_processor(value)
 
         self.set_arg_value(option, value)
 
@@ -100,20 +103,19 @@ class HotdocTargetBuilder:
             # When an option expects a single value, the unambiguous way
             # to specify it is with =
             if isinstance(value, str):
-                self.cmd.extend(['%s=%s' % (option, value)])
+                self.cmd.extend([f'{option}={value}'])
             else:
                 self.cmd.extend([option, value])
 
     def check_extra_arg_type(self, arg, value):
-        value = getattr(value, 'held_object', value)
         if isinstance(value, list):
             for v in value:
                 self.check_extra_arg_type(arg, v)
             return
 
-        valid_types = (str, bool, mesonlib.File, build.IncludeDirs, build.CustomTarget, build.BuildTarget)
+        valid_types = (str, bool, mesonlib.File, build.IncludeDirs, build.CustomTarget, build.CustomTargetIndex, build.BuildTarget)
         if not isinstance(value, valid_types):
-            raise InvalidArguments('Argument "%s=%s" should be of type: %s.' % (
+            raise InvalidArguments('Argument "{}={}" should be of type: {}.'.format(
                 arg, value, [t.__name__ for t in valid_types]))
 
     def process_extra_args(self):
@@ -136,27 +138,17 @@ class HotdocTargetBuilder:
                     if force_list and not isinstance(value, list):
                         return [value], uvalue
                     return value, uvalue
-            raise MesonException("%s field value %s is not valid,"
-                                 " valid types are %s" % (argname, value,
-                                                          types))
+            raise MesonException(f"{argname} field value {value} is not valid,"
+                                 f" valid types are {types}")
         except KeyError:
             if mandatory:
-                raise MesonException("%s mandatory field not found" % argname)
+                raise MesonException(f"{argname} mandatory field not found")
 
             if default is not None:
                 return default, default
 
         return None, None
 
-    def setup_extension_paths(self, paths):
-        if not isinstance(paths, list):
-            paths = [paths]
-
-        for path in paths:
-            self.add_extension_paths([path])
-
-        return []
-
     def add_extension_paths(self, paths):
         for path in paths:
             if path in self._extra_extension_paths:
@@ -165,10 +157,6 @@ class HotdocTargetBuilder:
             self._extra_extension_paths.add(path)
             self.cmd.extend(["--extra-extension-path", path])
 
-    def process_extra_extension_paths(self):
-        self.get_value([list, str], 'extra_extensions_paths',
-                       default="", value_processor=self.setup_extension_paths)
-
     def replace_dirs_in_string(self, string):
         return string.replace("@SOURCE_ROOT@", self.sourcedir).replace("@BUILD_ROOT@", self.builddir)
 
@@ -176,11 +164,10 @@ class HotdocTargetBuilder:
         if self.hotdoc.run_hotdoc(['--has-extension=gi-extension']) != 0:
             return
 
-        value, _ = self.get_value([list, str], 'gi_c_source_roots', default=[], force_list=True)
+        value = self.kwargs.pop('gi_c_source_roots')
         value.extend([
-            os.path.join(self.state.environment.get_source_dir(),
-                         self.interpreter.subproject_dir, self.state.subproject),
-            os.path.join(self.state.environment.get_build_dir(), self.interpreter.subproject_dir, self.state.subproject)
+            os.path.join(self.sourcedir, self.state.root_subdir),
+            os.path.join(self.builddir, self.state.root_subdir)
         ])
 
         self.cmd += ['--gi-c-source-roots'] + value
@@ -188,9 +175,8 @@ class HotdocTargetBuilder:
     def process_dependencies(self, deps):
         cflags = set()
         for dep in mesonlib.listify(ensure_list(deps)):
-            dep = getattr(dep, "held_object", dep)
             if isinstance(dep, InternalDependency):
-                inc_args = get_include_args(dep.include_directories)
+                inc_args = self.state.get_include_args(dep.include_directories)
                 cflags.update([self.replace_dirs_in_string(x)
                                for x in inc_args])
                 cflags.update(self.process_dependencies(dep.libraries))
@@ -210,29 +196,28 @@ class HotdocTargetBuilder:
                 self.add_include_path(os.path.join(self.builddir, dep.hotdoc_conf.subdir))
                 self.cmd += ['--extra-assets=' + p for p in dep.extra_assets]
                 self.add_extension_paths(dep.extra_extension_paths)
-            elif isinstance(dep, build.CustomTarget) or isinstance(dep, build.BuildTarget):
+            elif isinstance(dep, (build.CustomTarget, build.BuildTarget)):
                 self._dependencies.append(dep)
+            elif isinstance(dep, build.CustomTargetIndex):
+                self._dependencies.append(dep.target)
 
         return [f.strip('-I') for f in cflags]
 
     def process_extra_assets(self):
-        self._extra_assets, _ = self.get_value("--extra-assets", (str, list), default=[],
-                                               force_list=True)
+        self._extra_assets = self.kwargs.pop('extra_assets')
+
         for assets_path in self._extra_assets:
             self.cmd.extend(["--extra-assets", assets_path])
 
     def process_subprojects(self):
-        _, value = self.get_value([
-            list, HotdocTarget], argname="subprojects",
-            force_list=True, value_processor=self.process_dependencies)
+        value = self.kwargs.pop('subprojects')
 
-        if value is not None:
-            self._subprojects.extend(value)
+        self.process_dependencies(value)
+        self._subprojects.extend(value)
 
     def flatten_config_command(self):
         cmd = []
         for arg in mesonlib.listify(self.cmd, flatten=True):
-            arg = getattr(arg, 'held_object', arg)
             if isinstance(arg, mesonlib.File):
                 arg = arg.absolute_path(self.state.environment.get_source_dir(),
                                         self.state.environment.get_build_dir())
@@ -242,9 +227,12 @@ class HotdocTargetBuilder:
                     cmd.append(os.path.join(self.builddir, arg.get_curdir(), inc_dir))
 
                 continue
-            elif isinstance(arg, build.CustomTarget) or isinstance(arg, build.BuildTarget):
+            elif isinstance(arg, (build.BuildTarget, build.CustomTarget)):
                 self._dependencies.append(arg)
                 arg = self.interpreter.backend.get_target_filename_abs(arg)
+            elif isinstance(arg, build.CustomTargetIndex):
+                self._dependencies.append(arg.target)
+                arg = self.interpreter.backend.get_target_filename_abs(arg)
 
             cmd.append(arg)
 
@@ -265,7 +253,7 @@ class HotdocTargetBuilder:
                 res.append(self.ensure_file(val))
             return res
 
-        if not isinstance(value, mesonlib.File):
+        if isinstance(value, str):
             return mesonlib.File.from_source_file(self.sourcedir, self.subdir, value)
 
         return value
@@ -277,43 +265,40 @@ class HotdocTargetBuilder:
             _dir = os.path.join(self.sourcedir, self.subdir, value)
 
         if not os.path.isdir(_dir):
-            raise InvalidArguments('"%s" is not a directory.' % _dir)
+            raise InvalidArguments(f'"{_dir}" is not a directory.')
 
         return os.path.relpath(_dir, os.path.join(self.builddir, self.subdir))
 
-    def check_forbiden_args(self):
+    def check_forbidden_args(self):
         for arg in ['conf_file']:
             if arg in self.kwargs:
-                raise InvalidArguments('Argument "%s" is forbidden.' % arg)
+                raise InvalidArguments(f'Argument "{arg}" is forbidden.')
 
     def add_include_path(self, path):
         self.include_paths[path] = path
 
     def make_targets(self):
-        self.check_forbiden_args()
-        file_types = (str, mesonlib.File)
-        self.process_known_arg("--index", file_types, mandatory=True, value_processor=self.ensure_file)
-        self.process_known_arg("--project-version", str, mandatory=True)
-        self.process_known_arg("--sitemap", file_types, mandatory=True, value_processor=self.ensure_file)
-        self.process_known_arg("--html-extra-theme", str, value_processor=self.ensure_dir)
-        self.process_known_arg(None, list, "include_paths", force_list=True,
-                               value_processor=lambda x: [self.add_include_path(self.ensure_dir(v)) for v in ensure_list(x)])
-        self.process_known_arg('--c-include-directories',
-                               [Dependency, build.StaticLibrary, build.SharedLibrary, list], argname="dependencies",
-                               force_list=True, value_processor=self.process_dependencies)
+        self.check_forbidden_args()
+        self.process_known_arg("--index", value_processor=self.ensure_file)
+        self.process_known_arg("--project-version")
+        self.process_known_arg("--sitemap", value_processor=self.ensure_file)
+        self.process_known_arg("--html-extra-theme", value_processor=self.ensure_dir)
+        self.process_known_arg(None, "include_paths",
+                               value_processor=lambda x: [self.add_include_path(self.ensure_dir(v)) for v in x])
+        self.process_known_arg('--c-include-directories', argname="dependencies", value_processor=self.process_dependencies)
         self.process_gi_c_source_roots()
         self.process_extra_assets()
-        self.process_extra_extension_paths()
+        self.add_extension_paths(self.kwargs.pop('extra_extension_paths'))
         self.process_subprojects()
 
-        install, install = self.get_value(bool, "install", mandatory=False)
+        install = self.kwargs.pop('install')
         self.process_extra_args()
 
         fullname = self.name + '-doc'
         hotdoc_config_name = fullname + '.json'
         hotdoc_config_path = os.path.join(
             self.builddir, self.subdir, hotdoc_config_name)
-        with open(hotdoc_config_path, 'w') as f:
+        with open(hotdoc_config_path, 'w', encoding='utf-8') as f:
             f.write('{}')
 
         self.cmd += ['--conf-file', hotdoc_config_path]
@@ -326,8 +311,8 @@ class HotdocTargetBuilder:
         for path in self.include_paths.keys():
             self.cmd.extend(['--include-path', path])
 
-        if self.state.environment.coredata.get_builtin_option('werror'):
-            self.cmd.append('--fatal-warning')
+        if self.state.environment.coredata.get_option(mesonlib.OptionKey('werror', subproject=self.state.subproject)):
+            self.cmd.append('--fatal-warnings')
         self.generate_hotdoc_config()
 
         target_cmd = self.build_command + ["--internal", "hotdoc"] + \
@@ -337,26 +322,29 @@ class HotdocTargetBuilder:
         target = HotdocTarget(fullname,
                               subdir=self.subdir,
                               subproject=self.state.subproject,
+                              environment=self.state.environment,
                               hotdoc_conf=mesonlib.File.from_built_file(
                                   self.subdir, hotdoc_config_name),
                               extra_extension_paths=self._extra_extension_paths,
                               extra_assets=self._extra_assets,
                               subprojects=self._subprojects,
                               command=target_cmd,
-                              depends=self._dependencies,
-                              output=fullname,
+                              extra_depends=self._dependencies,
+                              outputs=[fullname],
+                              sources=[],
                               depfile=os.path.basename(depfile),
                               build_by_default=self.build_by_default)
 
         install_script = None
-        if install is True:
-            install_script = HotdocRunScript(self.build_command, [
+        if install:
+            install_script = self.state.backend.get_executable_serialisation(self.build_command + [
                 "--internal", "hotdoc",
                 "--install", os.path.join(fullname, 'html'),
                 '--name', self.name,
                 '--builddir', os.path.join(self.builddir, self.subdir)] +
                 self.hotdoc.get_command() +
                 ['run', '--conf-file', hotdoc_config_name])
+            install_script.tag = 'doc'
 
         return (target, install_script)
 
@@ -371,13 +359,13 @@ class HotdocTargetHolder(CustomTargetHolder):
     def config_path_method(self, *args, **kwargs):
         conf = self.held_object.hotdoc_conf.absolute_path(self.interpreter.environment.source_dir,
                                                           self.interpreter.environment.build_dir)
-        return self.interpreter.holderify(conf)
+        return conf
 
 
 class HotdocTarget(build.CustomTarget):
     def __init__(self, name, subdir, subproject, hotdoc_conf, extra_extension_paths, extra_assets,
-                 subprojects, **kwargs):
-        super().__init__(name, subdir, subproject, kwargs, absolute_paths=True)
+                 subprojects, environment, **kwargs):
+        super().__init__(name, subdir, subproject, environment, **kwargs, absolute_paths=True)
         self.hotdoc_conf = hotdoc_conf
         self.extra_extension_paths = extra_extension_paths
         self.extra_assets = extra_assets
@@ -391,13 +379,10 @@ class HotdocTarget(build.CustomTarget):
         return res
 
 
-class HotdocRunScript(build.RunScript):
-    def __init__(self, script, args):
-        super().__init__(script, args)
-
-
 class HotDocModule(ExtensionModule):
-    @FeatureNew('Hotdoc Module', '0.48.0')
+
+    INFO = ModuleInfo('hotdoc', '0.48.0')
+
     def __init__(self, interpreter):
         super().__init__(interpreter)
         self.hotdoc = ExternalProgram('hotdoc')
@@ -408,23 +393,44 @@ class HotDocModule(ExtensionModule):
             from hotdoc.run_hotdoc import run  # noqa: F401
             self.hotdoc.run_hotdoc = run
         except Exception as e:
-            raise MesonException('hotdoc %s required but not found. (%s)' % (
-                MIN_HOTDOC_VERSION, e))
+            raise MesonException(f'hotdoc {MIN_HOTDOC_VERSION} required but not found. ({e})')
+        self.methods.update({
+            'has_extensions': self.has_extensions,
+            'generate_doc': self.generate_doc,
+        })
 
     @noKwargs
+    @typed_pos_args('hotdoc.has_extensions', varargs=str, min_varargs=1)
     def has_extensions(self, state, args, kwargs):
-        res = self.hotdoc.run_hotdoc(['--has-extension=%s' % extension for extension in args]) == 0
-        return ModuleReturnValue(res, [res])
+        return self.hotdoc.run_hotdoc([f'--has-extension={extension}' for extension in args[0]]) == 0
 
+    @typed_pos_args('hotdoc.generate_doc', str)
+    @typed_kwargs(
+        'hotdoc.generate_doc',
+        KwargInfo('sitemap', file_types, required=True),
+        KwargInfo('index', file_types, required=True),
+        KwargInfo('project_version', str, required=True),
+        KwargInfo('html_extra_theme', (str, NoneType)),
+        KwargInfo('include_paths', ContainerTypeInfo(list, str), listify=True, default=[]),
+        # --c-include-directories
+        KwargInfo(
+            'dependencies',
+            ContainerTypeInfo(list, (Dependency, build.StaticLibrary, build.SharedLibrary)),
+            listify=True,
+            default=[],
+        ),
+        KwargInfo('gi_c_source_roots', ContainerTypeInfo(list, str), listify=True, default=[]),
+        KwargInfo('extra_assets', ContainerTypeInfo(list, str), listify=True, default=[]),
+        KwargInfo('extra_extension_paths', ContainerTypeInfo(list, str), listify=True, default=[]),
+        KwargInfo('subprojects', ContainerTypeInfo(list, HotdocTarget), listify=True, default=[]),
+        KwargInfo('install', bool, default=False),
+        allow_unknown=True
+    )
     def generate_doc(self, state, args, kwargs):
-        if len(args) != 1:
-            raise MesonException('One positional argument is'
-                                 ' required for the project name.')
-
         project_name = args[0]
         builder = HotdocTargetBuilder(project_name, state, self.hotdoc, self.interpreter, kwargs)
         target, install_script = builder.make_targets()
-        targets = [HotdocTargetHolder(target, self.interpreter)]
+        targets = [target]
         if install_script:
             targets.append(install_script)
 
@@ -432,4 +438,6 @@ class HotDocModule(ExtensionModule):
 
 
 def initialize(interpreter):
-    return HotDocModule(interpreter)
+    mod = HotDocModule(interpreter)
+    mod.interpreter.append_holder_map(HotdocTarget, HotdocTargetHolder)
+    return mod
