@@ -471,7 +471,7 @@ def _compare_output(expected: T.List[T.Dict[str, str]], output: str, desc: str)
                     sub = r'\\'
                 else:
                     sub = r'\\\\'
-                expected_line = re.sub(r'/(?=.*(WARNING|ERROR))', sub, expected_line)
+                expected_line = re.sub(r'/(?=.*(WARNING|ERROR|DEPRECATION))', sub, expected_line)
 
             m = OutputMatch(how, expected_line, count)
             if count == 0:
@@ -635,7 +635,7 @@ def _run_test(test: TestDef,
               should_fail: str) -> TestResult:
     gen_start = time.time()
     # Configure in-process
-    gen_args = []  # type: T.List[str]
+    gen_args = ['setup']
     if 'prefix' not in test.do_not_set_opts:
         gen_args += ['--prefix', 'x:/usr'] if mesonlib.is_windows() else ['--prefix', '/usr']
     if 'libdir' not in test.do_not_set_opts:
@@ -1059,7 +1059,8 @@ def detect_tests_to_run(only: T.Dict[str, T.List[str]], use_tmp: bool) -> T.List
                        shutil.which('flang') or
                        shutil.which('pgfortran') or
                        shutil.which('nagfor') or
-                       shutil.which('ifort'))
+                       shutil.which('ifort') or
+                       shutil.which('ifx'))
 
     skip_cmake = ((os.environ.get('compiler') == 'msvc2015' and under_ci) or
                   'cmake' not in tool_vers_map or
@@ -1245,7 +1246,7 @@ def _run_tests(all_tests: T.List[T.Tuple[str, T.List[TestDef], bool]],
     # one LogRunFuture and one TestRunFuture
     global safe_print
     futures_iter: T.Iterable[RunFutureUnion] = futures
-    if len(futures) > 2:
+    if len(futures) > 2 and sys.stdout.isatty():
         try:
             from tqdm import tqdm
             futures_iter = tqdm(futures, desc='Running tests', unit='test')
